<#
.NOTES
	Name: Check-SMTPTLSProtocol.ps1
	Author: Thomas Windscheif / excITe Consulting (twindscheif@excite-consulting.de)
	Requires: PowerShell v3 or higher and the Active Directory module
	Homepage: http://www.microlinc.de/
	Version History:
	1.1 - 07/23/2022 - added TLS 1.3-Protocol; fixing some errors in reporting
	1.0 - 12/07/2018 - Initial Release
.SYNOPSIS
	Tries to send several mails with different TLS protocols and returns the results.
.DESCRIPTION
	This Script uses the .NET Mailclass and tries to send mails to test the 5 available TLS protocols up from .NET Framework 4.5.x.
	The result is an object which contains the results of each test with the corresponding error message (if available).
.PARAMETER MailServerPort
	The Parameter defines the serverport. Normally port 25 or 587.
	It expects an integer value
	Default: 25
.PARAMETER MailServer
	This Parameter defines the FQDN or IP-Address of the SMTP-Server to be tested.
	e.g. smtp.office365.com for a test against the Exchange Online SMTP-Service.
	This is a mandatory parameter. Expects String.
.PARAMETER MailSender
	This Parameter defines the emailaddress of the Sender. It is used for "FROM"-field and "Reply-to"-Header
	e.g. yourname@yourdomain.com
	This is a mandatory parameter. Expects String.
.PARAMETER MailRecipient
	This Parameter defines the emailaddress of the Recipient. It is used for "TO"-field.
	e.g. anothername@foreigndomain.com
	This is a mandatory parameter. Expects String.
.PARAMETER MailSubject
	This Parameter defines the first part of the Message Subject. The script automatically appends the TLS protocol-Version which is tested.
	Expects String.
	Default: TLS-Test
.PARAMETER MailServerUser
	This Parameter defines the Username for the Credentials.
	Expects String
	e.g. yourname@yourdomain.com or YOURDOMAIN\yourname
	This is a optional parameter.
.PARAMETER MailServerPassword
	This Parameter defines the Password for the Credentials.
	It is expected to be typed in Cleartext. If you set username but no password you are prompted to type in password.
	Expects String
	This is a optional parameter.
.PARAMETER MailIgnoreTLSValidation
	This Parameter forces PowerShell to ignore possible invalid certificates (e.g. selfsigned)
	Expects boolean value
	Default: $false
	
.EXAMPLE
	[PS] C:\>.\Check-SMTPTLSProtocols.ps1 -MailServer smtp.office365.com -MailSender yourname@yourdomain.com -MailRecipient anothername@foreigndomain.com -MailServerUsername yourname@yourdomain.com
.EXAMPLE
	[PS] C:\>.\Check-SMTPTLSProtocols.ps1 -MailServer mail01.domain.local -MailSender yourname@yourdomain.com -MailRecipient yourname@yourdomain.com
.EXAMPLE
	[PS] C:\>.\Check-SMTPTLSProtocols.ps1 -MailServer mail01.domain.local -MailServerPort 587 -MailSender yourname@yourdomain.com -MailRecipient yourname@yourdomain.com
#>

[CmdletBinding()]
Param(
[int]$MailServerPort = 25,
[Parameter(Mandatory=$true)]
[string]$MailServer,
[Parameter(Mandatory=$true)]
[string]$MailSender,
[Parameter(Mandatory=$true)]
[string]$MailRecipient,
[string]$MailSubject = "TLS-Test",
[string]$MailServerUsername,
[string]$MailServerPassword,
$MailIgnoreTLSValidation = $false
)


#currently available Security Protocols can be retrieved through this cmd
#Source: https://psdevopsug.scot/post/force-powershell-to-use-tls-1-2/
#[enum]::GetNames([Net.SecurityProtocolType])

$SecurityProtocols = 	@('None',
			'[System.Net.ServicePointManager]::SecurityProtocol = [System.Net.SecurityProtocolType]::Tls13',
			'[System.Net.ServicePointManager]::SecurityProtocol = [System.Net.SecurityProtocolType]::Tls12',
			'[System.Net.ServicePointManager]::SecurityProtocol = [System.Net.SecurityProtocolType]::Tls11',
			'[System.Net.ServicePointManager]::SecurityProtocol = [System.Net.SecurityProtocolType]::Tls',
			'[System.Net.ServicePointManager]::SecurityProtocol = [System.Net.SecurityProtocolType]::Ssl3')

$ResultArray = @()
Foreach ($SecurityProtocol in $SecurityProtocols) {
	$ResultHash = $Null
	if ($SecurityProtocol -ne 'None') {
		$RegexedSecProtocol = "$($SecurityProtocol)" | Select-String -Pattern "::([A-Za-z0-9]*)$" -AllMatches
		$CurrentSecurityProtocol = $RegexedSecProtocol.Matches.Groups[1].Value
		try {
			Invoke-Expression "$SecurityProtocol"
		} catch {
			Write-Error "Error while Setting procotol $($CurrentSecurityProtocol) as TLS-Protocol. $($_.Exception.Message)"
			$ResultHash = @{
					TLSProtocol		= "$CurrentSecurityProtocol"
					TargetMailServer	= "$MailServer"
					Result			= "failed"
					Message 		= "cannot set procotol $($CurrentSecurityProtocol) as TLS protocol. Maybe an old .NET Framework version?"				
			}
			$ResultArray += New-Object PSObject -Property $ResultHash
		continue
		}
	} else {
		$CurrentSecurityProtocol = 'None'
	}
	Write-Host "Trying to send mail via TLS procotol $CurrentSecurityProtocol"
	try {
		$MailMessageObject = New-Object Net.Mail.MailMessage
		$MailObjectSmtp = New-Object Net.Mail.SmtpClient($MailServer, $MailServerPort)
		$MailMessageObject.From = "$MailSender"
		$MailObjectSmtp.UseDefaultCredentials = $false
		if ($MailServerUsername -ne "" -and $MailServerUsername -ne $null) {
			if ($MailServerPassword -ne "" -and $MailServerPassword -ne $null -and -not($PromptCredential)) {
				$SMTPCredentials = New-Object System.Net.NetworkCredential($MailServerUsername, $MailServerPassword)
			} else {
				if (-not($PromptCredential)) {
					$PromptCredential = Get-Credential -Username "$($MailServerUsername)" -Message SMTP-Credentials
					$PromptCredentialUsername = $PromptCredential.Username
					$PromptCredentialPassword = [System.Runtime.InteropServices.marshal]::PtrToStringAuto([System.Runtime.InteropServices.marshal]::SecureStringToBSTR($PromptCredential.Password))
				}
				$SMTPCredentials = New-Object System.Net.NetworkCredential("$PromptCredentialUsername", "$PromptCredentialPassword")
			}
			$MailObjectSmtp.Credentials = $SMTPCredentials
		}
		if ($SecurityProtocol -ne 'None') {
			$MailObjectSmtp.EnableSsl = $true
			if ($MailIgnoreTLSValidation) {
				Log -Errorlevel Warn -Message "Certificate Validation is set to be ignored!"
				## Code From http://poshcode.org/624
				## Create a compilation environment
				$Provider=New-Object Microsoft.CSharp.CSharpCodeProvider
				$Compiler=$Provider.CreateCompiler()
				$Params=New-Object System.CodeDom.Compiler.CompilerParameters
				$Params.GenerateExecutable=$False
				$Params.GenerateInMemory=$True
				$Params.IncludeDebugInformation=$False
				$Params.ReferencedAssemblies.Add("System.DLL") | Out-Null
				$TASource=
@'
				 namespace Local.ToolkitExtensions.Net.CertificatePolicy{
					 public class TrustAll : System.Net.ICertificatePolicy {
						 public TrustAll() {}
						public bool CheckValidationResult(System.Net.ServicePoint sp,
							 System.Security.Cryptography.X509Certificates.X509Certificate cert, 
							 System.Net.WebRequest req, int problem) {return true;}
					}
				 } 
'@ 
				$TAResults=$Provider.CompileAssemblyFromSource($Params,$TASource)
				$TAAssembly=$TAResults.CompiledAssembly
				## We now create an instance of the TrustAll and attach it to the ServicePointManager
				$TrustAll=$TAAssembly.CreateInstance("Local.ToolkitExtensions.Net.CertificatePolicy.TrustAll")
				[System.Net.ServicePointManager]::CertificatePolicy=$TrustAll
				## end code from http://poshcode.org/624
				[System.Net.ServicePointManager]::ServerCertificateValidationCallback = {return $true}
			}
		}
    		$MailMessageObject.ReplyTo = "$MailSender"
		$MailMessageObject.To.Add("$MailRecipient")
    		$MailMessageObject.Subject = "$MailSubject TLS-Protocol: $($CurrentSecurityProtocol)"
    		$MailMessageObject.Body = "Test Mail"
		$ResultHash = @{
				TLSProtocol		= "$CurrentSecurityProtocol";
				TargetMailServer 	= "$MailServer";
				}
    		$MailObjectSmtp.Send($MailMessageObject)
		$ResultHash += @{
				Result			= "success";
				Message			= "successfully sent message";
				}
	} catch {
		if ($_.Exception.InnerException.InnerException) {
			$ResultHash += @{
					Result			= "failed";
					Message			= "$($_.Exception.InnerException.InnerException.Message)";
					}
		} else {
			$ResultHash += @{
					Result			= "failed";
					Message			= "$($_.Exception.Message)";
					}
		}
	}
	$ResultArray += New-Object PSObject -Property $ResultHash
	Start-Sleep -Seconds 3
}

return $ResultArray
